@@ -7,26 +7,50 @@
 #include <QDebug>
 #include <QDir>
 #include <QLoggingCategory>
-#include <QQuickWebEngineProfile>
 #include <QStandardPaths>
+#include <QStringBuilder>
 
 #include "adblockfilterlistsmanager.h"
 
-#ifdef BUILD_ADBLOCK
 #include "angelfishsettings.h"
-#endif
+
+namespace ranges = std::ranges;
 
 Q_LOGGING_CATEGORY(AdblockCategory, "org.kde.angelfish.adblock", QtWarningMsg);
 
+#ifdef BUILD_ADBLOCK
+template <typename T>
+auto toRustType(T input) {
+    if constexpr (std::is_same_v<T, std::vector<QString>>) {
+        std::vector<rust::String> rustStringVec;
+        ranges::transform(input, std::back_inserter(rustStringVec), [](const QString &c) {
+            return rust::String(c.toStdString());
+        });
+        return rustStringVec;
+    }
+}
+
+template <typename T>
+auto toQtType(T input) {
+    if constexpr (std::is_same_v<T, rust::Vec<rust::String>>) {
+        std::vector<QString> qStringVec;
+        ranges::transform(input, std::back_inserter(qStringVec), [](const auto &c) {
+            return QString::fromStdString(std::string(c));
+        });
+        return qStringVec;
+    }
+}
+#endif
+
 AdblockUrlInterceptor::AdblockUrlInterceptor(QObject *parent)
     : QWebEngineUrlRequestInterceptor(parent)
 #ifdef BUILD_ADBLOCK
     // parsing the block lists takes some time, try to do it asynchronously
     // if it is not ready when it's needed, reading the future will block
-    , m_adblockInitFuture(std::async(std::launch::async, [this] { return createAdblock(); }))
+    , m_adblockInitFuture(std::async(std::launch::async, [this] { return createOrRestoreAdblock(); }))
     , m_adblock(std::nullopt)
-    , m_enabled(AngelfishSettings::adblockEnabled())
 #endif
+    , m_enabled(AngelfishSettings::adblockEnabled())
 {
 #ifdef BUILD_ADBLOCK
     connect(this, &AdblockUrlInterceptor::adblockInitialized, this, [this] {
@@ -39,12 +63,25 @@ AdblockUrlInterceptor::AdblockUrlInterceptor(QObject *parent)
 }
 
 #ifdef BUILD_ADBLOCK
-rust::Box<Adblock> AdblockUrlInterceptor::createAdblock()
+rust::Box<Adblock> AdblockUrlInterceptor::createOrRestoreAdblock()
 {
-    auto adb = new_adblock(AdblockFilterListsManager::filterListPath().toStdString());
+    rust::Box<Adblock> adb = [] {
+        auto cacheLocation = adblockCacheLocation();
+        if (QFile::exists(cacheLocation)) {
+            return loadAdblock(cacheLocation.toStdString());
+        }
+        return newAdblock(AdblockFilterListsManager::filterListPath().toStdString());
+    }();
+
     Q_EMIT adblockInitialized();
     return adb;
 }
+#endif
+
+QString AdblockUrlInterceptor::adblockCacheLocation()
+{
+    return QStandardPaths::writableLocation(QStandardPaths::CacheLocation) % u"/adblockCache";
+}
 
 bool AdblockUrlInterceptor::enabled() const
 {
@@ -56,7 +93,6 @@ void AdblockUrlInterceptor::setEnabled(bool enabled)
     m_enabled = enabled;
     AngelfishSettings::setAdblockEnabled(enabled);
 }
-#endif
 
 AdblockUrlInterceptor &AdblockUrlInterceptor::instance()
 {
@@ -65,6 +101,15 @@ AdblockUrlInterceptor &AdblockUrlInterceptor::instance()
     return instance;
 }
 
+AdblockUrlInterceptor::~AdblockUrlInterceptor()
+{
+#ifdef BUILD_ADBLOCK
+    if (m_adblock && (*m_adblock)->isValid() && (*m_adblock)->needsSave()) {
+        (*m_adblock)->save(adblockCacheLocation().toStdString());
+    }
+#endif
+}
+
 bool AdblockUrlInterceptor::downloadNeeded() const
 {
     return QDir(AdblockFilterListsManager::filterListPath()).isEmpty();
@@ -76,11 +121,32 @@ void AdblockUrlInterceptor::resetAdblock()
     if (m_adblock) {
         m_adblock = std::nullopt;
     }
-    m_adblockInitFuture = std::async(std::launch::async, [this] { return createAdblock(); });
+    m_adblockInitFuture = std::async(std::launch::async, [this] {
+        auto adb = newAdblock(AdblockFilterListsManager::filterListPath().toStdString());
+        Q_EMIT adblockInitialized();
+        return adb;
+    });
 #endif
 }
 
-inline std::string resourceTypeToString(const QWebEngineUrlRequestInfo::ResourceType type)
+#ifdef BUILD_ADBLOCK
+std::vector<QString> AdblockUrlInterceptor::getCosmeticFilters(const QUrl &url,
+                                                               const std::vector<QString> &classes,
+                                                               const std::vector<QString> &ids) const
+{
+    if (!m_adblock.has_value()) {
+        return std::vector<QString>();
+    }
+
+    const auto rustClasses = toRustType(classes);
+    const auto rustIds = toRustType(ids);
+    return toQtType((*m_adblock)->getCosmeticFilters(url.toString().toStdString(),
+                                                     {rustClasses.data(), rustClasses.size()},
+                                                     {rustIds.data(), rustIds.size()}));
+}
+#endif
+
+inline auto resourceTypeToString(const QWebEngineUrlRequestInfo::ResourceType type)
 {
     // Strings from https://docs.rs/crate/adblock/0.3.3/source/src/request.rs
     using Type = QWebEngineUrlRequestInfo::ResourceType;
@@ -131,11 +197,11 @@ void AdblockUrlInterceptor::interceptRequest(QWebEngineUrlRequestInfo &info)
 
     const std::string url = info.requestUrl().toString().toStdString();
     const std::string firstPartyUrl = info.firstPartyUrl().toString().toStdString();
-    const AdblockResult result = m_adblock.value()->should_block(url, firstPartyUrl, resourceTypeToString(info.resourceType()));
+    const AdblockResult result = (*m_adblock)->shouldBlock(url, firstPartyUrl, resourceTypeToString(info.resourceType()));
 
     const auto &redirect = result.redirect;
-    if (redirect.begin() != redirect.end()) {
-        info.redirect(QUrl(QString::fromStdString({redirect.begin(), redirect.end()})));
+    if (!redirect.empty()) {
+        info.redirect(QUrl(QString::fromStdString(std::string(redirect))));
     } else {
         info.block(result.matched);
     }
