@@ -1,5 +1,6 @@
 /*
     SPDX-FileCopyrightText: 2019 Jonah Brüchert <jbb.prv@gmx.de>
+    SPDX-FileCopyrightText: 2021 Carson Black <uhhadd@gmail.com>
 
     SPDX-License-Identifier: LGPL-2.0-or-later
 */
@@ -57,6 +58,7 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     QCoreApplication::setOrganizationDomain(QStringLiteral("kde.org"));
     QCoreApplication::setApplicationName(QStringLiteral("angelfish"));
     QCoreApplication::setApplicationVersion(QStringLiteral(ANGELFISH_VERSION_STRING));
+    QGuiApplication::setDesktopFileName(QStringLiteral("org.kde.angelfish"));
     KLocalizedString::setApplicationDomain("angelfish");
 
 #if QT_VERSION <= QT_VERSION_CHECK(5, 14, 0)
@@ -172,7 +174,21 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     }
 
     // Load QML
-    engine.load(QUrl(QStringLiteral("qrc:///webbrowser.qml")));
+
+    const QUrl url(SettingsHelper::isMobile()
+                   ? QStringLiteral("qrc:/mobile.qml")
+                   : QStringLiteral("qrc:/desktop.qml"));
+    QObject::connect(
+        &engine,
+        &QQmlApplicationEngine::objectCreated,
+        &app,
+        [url](QObject* obj, const QUrl& objUrl) {
+            if ((obj == nullptr) && url == objUrl) {
+                QCoreApplication::exit(-1);
+            }
+        },
+        Qt::QueuedConnection);
+    engine.load(url);
 
     // Error handling
     if (engine.rootObjects().isEmpty()) {
